﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Security;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Xml;
using Wintellect.WindowsAzure.Management;
using Wintellect.WindowsAzure.Secrets;

public static class AzureManagement {
    static string s_storageAccountLabel1 = "StagingAAAStagingAAAStagingAAAStagingAAAStagingAAAStagingAAAStagingZZZABCDE";
    static string s_storageAccountDesc1 = "Wintellect Demo Staging";
    static string s_storageAccountLocation1 = "West US";

    static string s_storageAccountName1 = "contosocohovinyard";
    static string s_storageAccountName2 = "contosotailspintoys";

    static string s_subscriptionId = AzureSecrets.ManagementSubscriptionId;
    static string s_certThumbprint = AzureSecrets.ManagementCertificateThumbprint;
    static string s_MsVersion = "2011-06-01";

    public static void CreateAccount(String storageAccountName, String label, String description, String affinityGroupName, String location) {

        Console.WriteLine("******************** Create Storage Account ********************\n");

        if ((affinityGroupName == null && location == null) || (affinityGroupName != null && location != null)) {
            throw new ArgumentException("affinityGroupName or location must be supplied (but not both).");
        }

        String subscriptionId = AzureSecrets.ManagementSubscriptionId;
        string operationName = "storageservices";
        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}";
        var requestUri = new Uri(String.Format(uriTemplate, subscriptionId, operationName));
        var encodedLabel = Convert.ToBase64String(Encoding.UTF8.GetBytes(label));

        Console.WriteLine("Length of encoded label is: " + encodedLabel.Length);

        XmlWriterSettings wSettings = new XmlWriterSettings();
        wSettings.Indent = true;
        MemoryStream ms = new MemoryStream();
        XmlWriter xw = XmlWriter.Create(ms, wSettings);

        xw.WriteStartDocument();
        xw.WriteStartElement("CreateStorageServiceInput", @"http://schemas.microsoft.com/windowsazure"); 
            xw.WriteStartElement("ServiceName"); xw.WriteString(storageAccountName); xw.WriteEndElement();
            xw.WriteStartElement("Description"); xw.WriteString(description); xw.WriteEndElement();
            xw.WriteStartElement("Label"); xw.WriteString(encodedLabel); xw.WriteEndElement();
            if (null != affinityGroupName) {
                xw.WriteStartElement("AffinityGroup");
                xw.WriteString(affinityGroupName);
                xw.WriteEndElement();
            }
            if (null != location) {
                xw.WriteStartElement("Location");
                xw.WriteString(location);
                xw.WriteEndElement();
            }
        xw.WriteEndElement();
        xw.WriteEndDocument();
        xw.Flush();

        byte[] bodyBytes = ms.ToArray();

        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "POST";
        request.ContentType = "application/xml";

        Stream dataStream = request.GetRequestStream();
        dataStream.Write(bodyBytes, 0, bodyBytes.Length);

        var requestStream = new MemoryStream();
        requestStream.Write(bodyBytes, 0, bodyBytes.Length);
        requestStream.Position = 0;

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, requestStream);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }
    }

    public static void GetAccountProperties(String storageAccountName) {
        Console.WriteLine("\n******************** Get Storage Account Properties ********************\n");

        string operationName = "storageservices";

        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}/{2}";
        var requestUri = new Uri(String.Format(uriTemplate, s_subscriptionId, operationName, storageAccountName));

        // Create the request and specify attributes of the request.
        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        // Define the required headers to specify the API version and operation type.
        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "GET";
        request.ContentType = "application/xml";

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, null);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }
    }

    public static void GetAccountKeys(String storageAccountName) {
        Console.WriteLine("\n******************** Get Storage Account Keys ********************\n");

        string operationName = "storageservices";

        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}/{2}/keys";
        var requestUri = new Uri(String.Format(uriTemplate, s_subscriptionId, operationName, storageAccountName));

        // Create the request and specify attributes of the request.
        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        // Define the required headers to specify the API version and operation type.
        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "GET";

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, null);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));

        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }
    }

    public static void RegenerateAccountKeys(String storageAccountName, String keyName) {
        Console.WriteLine("******************** Regenerate Storage Account Keys ********************\n");

        string operationName = "storageservices";
        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}/{2}/keys?action=regenerate";
        var requestUri = new Uri(String.Format(uriTemplate, s_subscriptionId, operationName, storageAccountName));

        XmlWriterSettings wSettings = new XmlWriterSettings();
        wSettings.Indent = true;
        MemoryStream ms = new MemoryStream();
        XmlWriter xw = XmlWriter.Create(ms, wSettings);

        xw.WriteStartDocument();
        xw.WriteStartElement("RegenerateKeys", @"http://schemas.microsoft.com/windowsazure");
        xw.WriteStartElement("KeyType"); xw.WriteString(keyName); xw.WriteEndElement();
        xw.WriteEndElement();
        xw.WriteEndDocument();
        xw.Flush();

        byte[] bodyBytes = ms.ToArray();

        //var sb = new StringBuilder();
        //sb.AppendFormat("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
        //sb.AppendFormat("<RegenerateKeys xmlns=\"http://schemas.microsoft.com/windowsazure\">");
        //sb.AppendFormat("  <KeyType>{0}</KeyType>", keyName);
        //sb.AppendFormat("</RegenerateKeys>");
        //byte[] bodyBytes = Encoding.UTF8.GetBytes(sb.ToString());

        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "POST";
        request.ContentType = "application/xml";

        Stream dataStream = request.GetRequestStream();
        dataStream.Write(bodyBytes, 0, bodyBytes.Length);

        var requestStream = new MemoryStream();
        requestStream.Write(bodyBytes, 0, bodyBytes.Length);
        requestStream.Position = 0;

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, requestStream);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }
    }

    public static void UpdateAccount(String storageAccountName, String label, String description) {
        Console.WriteLine("\n******************** Update Storage Account *******************\n");

        string operationName = "storageservices";
        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}/{2}";
        var requestUri = new Uri(String.Format(uriTemplate, s_subscriptionId, operationName, storageAccountName));

        XmlWriterSettings wSettings = new XmlWriterSettings();
        wSettings.Indent = true;
        MemoryStream ms = new MemoryStream();
        XmlWriter xw = XmlWriter.Create(ms, wSettings);

        xw.WriteStartDocument();
        xw.WriteStartElement("UpdateStorageServiceInput", @"http://schemas.microsoft.com/windowsazure");
        xw.WriteStartElement("Description"); xw.WriteString(description); xw.WriteEndElement();
        xw.WriteStartElement("Label"); xw.WriteString(label); xw.WriteEndElement();
        xw.WriteEndElement();
        xw.WriteEndDocument();
        xw.Flush();

        byte[] bodyBytes = ms.ToArray();

        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "PUT";
        request.ContentType = "application/xml";

        Stream dataStream = request.GetRequestStream();
        dataStream.Write(bodyBytes, 0, bodyBytes.Length);

        var requestStream = new MemoryStream();
        requestStream.Write(bodyBytes, 0, bodyBytes.Length);
        requestStream.Position = 0;

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, requestStream);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }

    }

    public static void DeleteAccount(String storageAccountName) {
        Console.WriteLine("\n******************** Delete Storage Account ********************\n");

        string operationName = "storageservices";
        String uriTemplate = "https://management.core.windows.net/{0}/services/{1}/{2}";
        var requestUri = new Uri(String.Format(uriTemplate, s_subscriptionId, operationName, storageAccountName));

        // Create the request and specify attributes of the request.
        HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(requestUri);

        // Define the required headers to specify the API version and operation type.
        request.Headers.Add("x-ms-version", s_MsVersion);
        request.Method = "DELETE";

        PrintStreamToConsole("---Request---", request.Method, requestUri.ToString(), String.Empty, request.Headers, null);
        HttpWebResponse response;
        // attach credentials used to authenticate the request
        request.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
        try {
            response = (HttpWebResponse)request.GetResponse();
            PrintStreamToConsole("--Response--", String.Empty, String.Empty, response.StatusDescription, response.Headers, response.GetResponseStream());
        }
        catch (Exception ex) {
            Console.WriteLine(ex.Message);
        }
    }

    public static String Base64Encode(String stringToEncode) {
        byte[] encodedAccountNameBytes = Encoding.UTF8.GetBytes(stringToEncode);
        return Convert.ToBase64String(encodedAccountNameBytes);
    }

    public static void PrintStreamToConsole(String title, string verb, String uri, string statusDescription, WebHeaderCollection headers, Stream stream) {
        Console.WriteLine(title);

        if (!String.IsNullOrEmpty(verb)) Console.WriteLine("{0} {1} HTTP/1.1", verb, uri);

        // Display the web response status code.
        if (!String.IsNullOrEmpty(statusDescription)) {
            Int32 code = (Int32)Enum.Parse(typeof(HttpStatusCode), statusDescription);
            Console.WriteLine("HTTP/1.1" + " " + code);
        }

        // print all of the response headers
        foreach (string header in headers) {
            Console.WriteLine(header + ": " + headers[header]);
        }
        if (stream != null) {
            //stream.Position = 0;
            using (StreamReader reader = new StreamReader(stream)) {
                Console.WriteLine(reader.ReadToEnd());
            }
        }
    }
}

