﻿using System;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Table;
using Microsoft.WindowsAzure.Storage.Table.DataServices;
using Wintellect.WindowsAzure.Storage.Blob;
using Wintellect.WindowsAzure.Storage.Table;

public static class AnalyticPatterns {

    public static void AnalyticsLogs(CloudStorageAccount account) {
        CloudBlobClient client = account.CreateCloudBlobClient();
        CloudBlobContainer c = client.GetContainerReference("$logs");
        for (BlobResultSegment brs = null; brs.HasMore(); ) {
            brs = c.ListBlobsSegmented(null, true, BlobListingDetails.None, 1000, brs.SafeContinuationToken(), null, null);
            foreach (var b in brs.Results) {
                Console.WriteLine(" " + b.Uri);
                CloudBlockBlob blob = c.GetBlockBlobReference(b.Uri.ToString());
                blob.FetchAttributes();
                foreach (String key in blob.Metadata.Keys) {
                    Console.WriteLine("\t{0}={1}", key, blob.Metadata[key]);
                }
            }
        }
    }

    public static void AnalyticsBlobMetrics(CloudStorageAccount account) {
        CloudTableClient client = account.CreateCloudTableClient();
        CloudTable ct = client.GetTableReference("$MetricsTransactionsBlob");
        var query = new TableQuery<MetricsTransactionBlobEntity>();

        Int32 count = 0;
        var qr = ct.ExecuteQuery(query);
        foreach (var entity in qr) {
            Console.WriteLine("PK={0}, RK={1}, TS={2}", entity.PartitionKey, entity.RowKey, entity.Timestamp);
            count++;
        }
        Console.WriteLine("Blob Transactions={0}", count);
        Console.WriteLine();
    }


    public static void AnalyticsCapacityBlob(CloudStorageAccount account) {
        CloudTableClient client = account.CreateCloudTableClient();
        CloudTable ct = client.GetTableReference("$MetricsCapacityBlob");
        TableQuery<MetricsCapacityBlobEntity> query = new TableQuery<MetricsCapacityBlobEntity>();
        Int32 count = 0;
        var qr = ct.ExecuteQuery(query);
        foreach (var entity in qr) {
            Console.WriteLine("PK={0}, RK={1}, TS={2}, Cap={3}, CC={4}, OC={5}",
                entity.PartitionKey, entity.RowKey, entity.Timestamp, entity.Capacity,
                entity.ContainerCount, entity.ObjectCount);
            count++;
        }
        Console.WriteLine("Blob Transactions={0}", count);
        Console.WriteLine();
    }

    public class MetricsTransactionBlobEntity : TableEntity {
        public String Time { get; set; }
        public String AccessTypeTransactionType { get; set; }
        public long TotalIngress { get; set; }
        public long TotalEgress { get; set; }
        public long TotalRequests { get; set; }
        public long TotalBillableRequests { get; set; }
        public long Availability { get; set; }
        public Double AverageE2ELatency { get; set; }
        public Double AverageServerLatency { get; set; }
        public Double PercentSuccess { get; set; }
        public Double PercentThrottlingError { get; set; }
        public Double PercentTimeoutError { get; set; }
        public Double PercentServerOtherError { get; set; }
        public Double PercentClientOtherError { get; set; }
        public Double PercentAuthorizationError { get; set; }
        public Double PercentNetworkError { get; set; }
        public long Success { get; set; }
        public long AnonymousSuccess { get; set; }
        public long SASSuccess { get; set; }
        public long ThrottlingError { get; set; }
        public long AnonymousThrottlingError { get; set; }
        public long SASThrottlingError { get; set; }
        public long ClientTimeoutError { get; set; }
        public long AnonymousClientTimeoutError { get; set; }
        public long SASClientTimeoutError { get; set; }
        public long ServerTimeoutError { get; set; }
        public long AnonymousServerTimeoutError { get; set; }
        public long SASServerTimeoutError { get; set; }
        public long ClientOtherError { get; set; }
        public long SASClientOtherError { get; set; }
        public long AnonymousClientOtherError { get; set; }
        public long ServerOtherError { get; set; }
        public long AnonymousServerOtherError { get; set; }
        public long SASServerOtherError { get; set; }
        public long AuthorizationError { get; set; }
        public long AnonymousAuthorizationError { get; set; }
        public long SASAuthorizationError { get; set; }
        public long NetworkError { get; set; }
        public long AnonymousNetworkError { get; set; }
        public long SASNetworkError { get; set; }
    }


    public class MetricsCapacityBlobEntity : TableEntity {
        public long Capacity { get; set; }
        public long ContainerCount { get; set; }
        public long ObjectCount { get; set; }
    }
}


