﻿/******************************************************************************
Module:  StoragePatterns.cs
Notices: Copyright (c) 2011 by Wintellect
******************************************************************************/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Xml.Linq;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Auth;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using Microsoft.WindowsAzure.Storage.Table;
using Wintellect.WindowsAzure.Management;
using Wintellect.WindowsAzure.Secrets;
using Wintellect.WindowsAzure.Storage;
using Wintellect.WindowsAzure.Storage.Blob;
using Wintellect.WindowsAzure.Storage.Queue;
using Wintellect.WindowsAzure.Storage.Table;

public static class AzureBookStoragePatterns {
   private static readonly Boolean c_SpawnStorageKiller = false;
   private static CloudStorageAccount m_account = null;
   private static CloudStorageAccount Account {
      //[DebuggerStepThrough]
      get {
         if (m_account != null) return m_account;
         // Chose the default account to execute these demos against:
         m_account =
            //GetStorageAccount(FiddlerPrompt(StorageAccountType.DevStorage, true));
            //GetStorageAccount(FiddlerPrompt(StorageAccountType.DevStorageWithFiddler, true));
            GetStorageAccount(FiddlerPrompt(StorageAccountType.AzureStorage, true));
         return m_account;
      }
   }

   private const String c_tableName = "Demo";

   static string s_storageAccountLabel1 = "StagingAAAStagingAAAStagingAAAStagingAAAStagingAAAStagingAAAStagingZZZABCDE";
   static string s_storageAccountDesc1 = "Wintellect Demo Staging";
   static string s_storageAccountLocation1 = "West US";

   static string s_storageAccountName1 = "contosocohovinyard";
   static string s_storageAccountName2 = "contosotailspintoys";

   static string s_subscriptionId = AzureSecrets.ManagementSubscriptionId;
   static string s_certThumbprint = AzureSecrets.ManagementCertificateThumbprint;
   static string s_MsVersion = "2011-06-01";

   public static void Main() {
      Management.Rest();
      StorageAccountsEndpoints();

      AzureManagement.CreateAccount(s_storageAccountName1, s_storageAccountLabel1, s_storageAccountDesc1, null, s_storageAccountLocation1);
      AzureManagement.GetAccountProperties(s_storageAccountName1);
      AzureManagement.GetAccountKeys(s_storageAccountName1);
      AzureManagement.RegenerateAccountKeys(s_storageAccountName1, "Primary");
      AzureManagement.UpdateAccount(s_storageAccountName1, Convert.ToBase64String(Encoding.UTF8.GetBytes(s_storageAccountName2)), "Label Changed");
      AzureManagement.GetAccountProperties(s_storageAccountName1);
      AzureManagement.DeleteAccount(s_storageAccountName1);

      BlobPatterns.Basics(Account);
      BlobPatterns.RootContainer(Account);
      BlobPatterns.Attributes(Account);
      BlobPatterns.ConditionalOperations(Account);
      BlobPatterns.SignedAccessSignatures(Account);
      BlobPatterns.BlockBlobs(Account);
      BlobPatterns.PageBlobs(Account);
      BlobPatterns.Snapshots(Account);
      BlobPatterns.Leases(Account);
      BlobPatterns.DirectoryHierarchies(Account);
      BlobPatterns.Segmented(Account);


      TablePatterns.Basics(Account);
      TablePatterns.OptimisticConcurrency(Account);
      TablePatterns.LastUpdateWins(Account);
      TablePatterns.QueryFilterStrings(Account);
      TablePatterns.Segmented(Account);
      TablePatterns.MultipleKinds(Account);

      QueuePatterns.Basics(Account);
      QueuePatterns.Segmented(Account);

      AnalyticPatterns.AnalyticsLogs(Account);
      AnalyticPatterns.AnalyticsBlobMetrics(Account);
      AnalyticPatterns.AnalyticsCapacityBlob(Account);

      Console.WriteLine("===== finished =====");
      Console.ReadLine();

      Console.ReadLine();

      StorageExtensions.ImprovePerformance(null, 100);
   }

   [DebuggerStepThrough]
   private static StorageAccountType FiddlerPrompt(StorageAccountType accountType, Boolean prompt = true) {
      const MessageBoxOptions MB_TOPMOST = (MessageBoxOptions)0x00040000;
      if (prompt && (accountType == StorageAccountType.DevStorage)) {
         if (MessageBox.Show(
          "Drag Fiddler's Process Filter cursor on this window.\n" +
          "Are you using Fiddler?",
          "Wintellect's Windows Azure Data Storage Demo",
          MessageBoxButtons.YesNo, MessageBoxIcon.Information,
          MessageBoxDefaultButton.Button1, MB_TOPMOST) == DialogResult.Yes)
            accountType = StorageAccountType.DevStorageWithFiddler;
      }
      return accountType;
   }

   public enum StorageAccountType {
      AzureStorage,
      DevStorage,
      DevStorageWithFiddler
   }

   [DebuggerStepThrough]
   private static CloudStorageAccount GetStorageAccount(StorageAccountType accountType) {
       switch (accountType) {
           default:
           case StorageAccountType.DevStorage:
               return CloudStorageAccount.DevelopmentStorageAccount;
           case StorageAccountType.DevStorageWithFiddler:
               return CloudStorageAccount.Parse("UseDevelopmentStorage=true;DevelopmentStorageProxyUri=http://ipv4.fiddler");
           case StorageAccountType.AzureStorage:
               String accountName = AzureSecrets.StorageAccountName;
               String accountKey = AzureSecrets.StorageAccountKey;
               return new CloudStorageAccount(new StorageCredentials(accountName, accountKey), false);
       }
   }

   private static void StorageAccountsEndpoints() {
       Console.Clear();

       Console.WriteLine("Azure storage endpoints:");
       String accountName = AzureSecrets.StorageAccountName;
       String accountKey = AzureSecrets.StorageAccountKey;
       CloudStorageAccount account = new CloudStorageAccount(new StorageCredentials(accountName, accountKey), true);
       Console.WriteLine("   BlobEndpoint:  " + account.BlobEndpoint);
       Console.WriteLine("   TableEndpoint: " + account.TableEndpoint);
       Console.WriteLine("   QueueEndpoint: " + account.QueueEndpoint);
       Console.WriteLine();

       Console.WriteLine("Storage emulator endpoints:");
       account = CloudStorageAccount.DevelopmentStorageAccount;
       Console.WriteLine("   BlobEndpoint:  " + account.BlobEndpoint);
       Console.WriteLine("   TableEndpoint: " + account.TableEndpoint);
       Console.WriteLine("   QueueEndpoint: " + account.QueueEndpoint);
       Console.WriteLine();
   }

   private static class Management {
       public static void Rest() {
           String subscriptionId = AzureSecrets.ManagementSubscriptionId;
           String serviceName = AzureSecrets.StorageAccountName;
           var req = (HttpWebRequest)
              WebRequest.Create(String.Format("https://management.core.windows.net/{0}/services/storageservices/{1}",
              subscriptionId, serviceName));
           req.Headers.Add("x-ms-version", "2009-10-01");
           req.ClientCertificates.Add(AzureManagementClient.FindCertificate((AzureSecrets.ManagementCertificateThumbprint)));
           using (var response = req.GetResponse()) {
               Console.WriteLine(XElement.Load(response.GetResponseStream()));
           }
       }
   }

   private static class BlobPatterns {
       public static void Basics(CloudStorageAccount account) {
           Console.Clear();

           // Use an OperationContext for debugging and to estimate billing
           OperationContext oc = new OperationContext();
           oc.SendingRequest += (Object s, RequestEventArgs e) => {
               HttpWebRequest request = e.Request;
           };
           oc.ResponseReceived += (Object s, RequestEventArgs e) => {
               HttpWebRequest request = e.Request;
               HttpWebResponse response = e.Response;
               RequestResult rr = e.RequestInformation;
           };

           CloudBlobClient client = account.CreateCloudBlobClient();

           // Create a container:
           CloudBlobContainer container = client.GetContainerReference("demo");
           Boolean created = container.CreateIfNotExists(null, oc);

           // Show this account's containers:
           foreach (var c in client.ListContainers(null, ContainerListingDetails.None, operationContext: oc))
               Console.WriteLine(c.Uri);
           Console.WriteLine();

           // Create 2 blobs in the container:
           CloudBlockBlob blob = container.GetBlockBlobReference("SomeData.txt");
           using (var stream = new MemoryStream(("Some data created at " + DateTime.Now).Encode())) {
               blob.UploadFromStream(stream, operationContext: oc);
           }
           using (var stream = new MemoryStream()) {
               blob.DownloadToStream(stream, operationContext: oc);
               stream.Seek(0, SeekOrigin.Begin);
               Console.WriteLine(new StreamReader(stream).ReadToEnd());   // Read the blob data back
           }

           blob = container.GetBlockBlobReference("SomeMoreData.txt");
           using (var stream = new MemoryStream(("Some more data created at " + DateTime.Now).Encode())) {
               blob.UploadFromStream(stream, operationContext: oc);
           }

           // List the blobs in container & show each blob's URIs:
           foreach (IListBlobItem b in container.ListBlobs(operationContext: oc))
               Console.WriteLine(b.Uri);
           Console.WriteLine();

           // Attempt to access a blob from browser (Fails due to security):
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit();// Fails

           // Attempt to access the blob anonymously in code:
           CloudBlobClient anonymous = new CloudBlobClient(client.BaseUri);
           var credentials = anonymous.Credentials;  // Prove we have no credentials
           try {
               using (var stream = new MemoryStream()) {
                   anonymous.GetContainerReference("demo").GetBlockBlobReference("SomeData.txt")
                      .DownloadToStream(stream, operationContext: oc);
                   Console.WriteLine("Download result: ", stream.GetBuffer().Decode());
                   Console.WriteLine();
               }
           }
           catch (StorageException se) {
               Console.WriteLine(se.Message);
               if ((HttpStatusCode)se.RequestInformation.HttpStatusCode != HttpStatusCode.NotFound) throw;
           }

           // Change container's security to allow read access to its blobs:
           BlobContainerPermissions permissions = new BlobContainerPermissions {
               PublicAccess = BlobContainerPublicAccessType.Container
           };
           container.SetPermissions(permissions, operationContext: oc);

           // Attempt to access a blob from browser & in code (succeeds):
           Process.Start("IExplore", container.Uri.ToString()).WaitForExit();
           using (var stream = new MemoryStream()) {
               anonymous.GetContainerReference("demo").GetBlockBlobReference("SomeData.txt")
                  .DownloadToStream(stream, operationContext: oc);
               Console.WriteLine("Download result: " + stream.GetBuffer().Decode());
               Console.WriteLine();
           }

           // Show the container's blobs via REST:
           Process.Start("IExplore", container.Uri + "?comp=list").WaitForExit();

           // Delete the container:
           container.Delete(operationContext: oc);

           // How many responses were made in this method?
           var numRequests = oc.RequestResults.Count();
       }

       public static void RootContainer(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();

           // Create $root container, set permissions, put blob in it, access without $root:
           CloudBlobContainer container = client.GetContainerReference("$root").EnsureExists();
           container.SetPermissions(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Container });
           CloudBlockBlob blob = container.GetBlockBlobReference("RootData.txt");
           var b = client.GetRootContainerReference().GetBlockBlobReference("RootData.txt").Uri.Equals(blob.Uri);
           using (var stream = new MemoryStream("This data is in a root blob".Encode())) {
               blob.UploadFromStream(stream);
           }
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit();    // With $root

           String rootBlobUri = blob.Uri.ToString().Replace("$root/", String.Empty);
           Process.Start("IExplore", rootBlobUri); // Without $root

           container.Delete();
       }

       public static void Attributes(CloudStorageAccount account) {
           Console.Clear();
           // Create a blob and attach some metadata to it:
           CloudBlobClient client = account.CreateCloudBlobClient();
           CloudBlobContainer container = client.GetContainerReference("demo").EnsureExists();
           container.SetPermissions(new BlobContainerPermissions() { PublicAccess = BlobContainerPublicAccessType.Container });
           CloudBlockBlob blob = container.GetBlockBlobReference("ReadMe.txt");
           using (var stream = new MemoryStream("This is some text".Encode())) {
               blob.UploadFromStream(stream);
           }
           blob.Metadata["CreatedBy"] = "Paul";
           blob.Metadata["SourceMachine"] = Environment.MachineName;
           blob.SetMetadata();
           // NOTE: SetMetadata & SetProperties update the blob's ETag & LastModifiedUtc

           // Get blobs in container showing each blob's properties & metadata
           // See http://msdn.microsoft.com/en-us/library/dd135734.aspx for more options
           Process.Start("IExplore", container.Uri + "?restype=container&comp=list&include=metadata");

           // This is the same as above but in code:
           container.ListBlobs(null, false, BlobListingDetails.Metadata);

           // Read the blob's attributes (which includes properties & metadata)
           blob.FetchAttributes();
           BlobProperties p = blob.Properties;
           Console.WriteLine("Blob's metadata (LastModified={0}, ETag={1})", p.LastModified, p.ETag);
           Console.WriteLine("   Content Type={0}, Encoding={1}, Language={2}, MD5={3}",
              p.ContentType, p.ContentEncoding, p.ContentLanguage, p.ContentMD5);
           Console.WriteLine();
           foreach (var kvp in blob.Metadata)
               Console.WriteLine("   {0} = {1}", kvp.Key, kvp.Value);
       }

       public static void ConditionalOperations(CloudStorageAccount account) {
           Console.Clear();
           // Create a blob and attach some metadata to it:
           CloudBlobClient client = account.CreateCloudBlobClient();

           // No retry for 306 (Unused), 4xx, 501 (NotImplemented), 505 (HttpVersionNotSupported)
           client.RetryPolicy = new ExponentialRetry();

           // Time server can process a request (default = 90 secs)
           client.ServerTimeout = TimeSpan.FromSeconds(90);

           // Time client can wait for response across all retries (default = disabled)
           client.MaximumExecutionTime = TimeSpan.FromSeconds(5);

           CloudBlobContainer container = client.GetContainerReference("demo").EnsureExists();
           CloudBlockBlob blob = container.GetBlockBlobReference("Data.txt");
           using (var stream = new MemoryStream("Data".Encode())) {
               blob.UploadFromStream(stream);
           }


           // Download blob content if newer than what we have:
           //Try(() => blob.DownloadText(AccessCondition.GenerateIfModifiedSinceCondition(blob.Properties.LastModified.Value)));  // Fails
           try {
               blob.DownloadText(AccessCondition.GenerateIfModifiedSinceCondition(blob.Properties.LastModified.Value)); // Fails
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",
                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           // Download blob content if more than 1 day old:
           //Try(() => blob.DownloadText(AccessCondition.GenerateIfNotModifiedSinceCondition(DateTimeOffset.Now.AddDays(-1))));  // Fails
           try {
               blob.DownloadText(AccessCondition.GenerateIfNotModifiedSinceCondition(DateTimeOffset.Now.AddDays(-1))); // Fails
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",
                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           // Upload new content if the blob wasn't changed behind your back:
           //Try(() => blob.UploadText("Succeeds", AccessCondition.GenerateIfMatchCondition(blob.Properties.ETag)));  // Succeeds
           try {
               blob.UploadText("Succeeds", AccessCondition.GenerateIfMatchCondition(blob.Properties.ETag)); // Succeeds
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",
                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           // Download content if it changed behind our back:
           //Try(() => blob.DownloadText(AccessCondition.GenerateIfNoneMatchCondition(blob.Properties.ETag)));       // Fails
           try {
               blob.DownloadText(AccessCondition.GenerateIfNoneMatchCondition(blob.Properties.ETag)); // Fails
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",

                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           // Upload your content if it doesn't already exist:
           //Try(() => blob.UploadText("Fails", AccessCondition.GenerateIfNoneMatchCondition("*"))); // Fails
           try {
               blob.UploadText("Fails", AccessCondition.GenerateIfNoneMatchCondition("*")); // Fails
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",

                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

       }

       private static void Try(Func<String> func) {
           try { Console.WriteLine(func()); }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",
                  (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                  ex.RequestInformation.HttpStatusMessage));
           }
       }

       private static void Try(Action action) {
           Try(() => { action(); return "Success"; });
       }

       public static void BlockBlobs(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();
           CloudBlobContainer container = client.GetContainerReference("demo").EnsureExists();
           CloudBlockBlob blockBlob = container.GetBlockBlobReference("MyBlockBlob.txt");
           blockBlob.DeleteIfExists();

           // Put 3 blocks to the blob verifying data integrity
           String[] words = new[] { "A ", "B ", "C " };
           var MD5 = new MD5Cng();
           for (Int32 word = 0; word < words.Length; word++) {
               Byte[] wordBytes = words[word].Encode(Encoding.UTF8);

               // Azure verifies data integrity during transport; failure=400 (Bad Request)
               String md5Hash = MD5.ComputeHash(wordBytes).ToBase64String();
               blockBlob.PutBlock(word.ToBlockId(), new MemoryStream(wordBytes), md5Hash);
           }

           Console.WriteLine("Blob's uncommitted blocks:");
           foreach (ListBlockItem lbi in blockBlob.DownloadBlockList(BlockListingFilter.Uncommitted))
               Console.WriteLine("   Name={0}, Length={1}, Committed={2}",
                  lbi.Name.FromBlockId(), lbi.Length, lbi.Committed);
           // Fails
           try {
               blockBlob.DownloadText();
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",
                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           // Commit the blocks in order (and multiple times):
           blockBlob.PutBlockList(new[] { 0.ToBlockId(), 0.ToBlockId(), 1.ToBlockId(), 2.ToBlockId() });
           // Succeeds
           try {
               blockBlob.DownloadText();
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",

                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }

           Console.WriteLine("Blob's committed blocks:");
           foreach (ListBlockItem lbi in blockBlob.DownloadBlockList())
               Console.WriteLine("   Name={0}, Length={1}, Committed={2}",
                  lbi.Name.FromBlockId(), lbi.Length, lbi.Committed);
           
           // You can change the block order & remove a block:
           blockBlob.PutBlockList(new[] { 2.ToBlockId(), 1.ToBlockId() });
           // Succeeds
           try {
               blockBlob.DownloadText();
           }
           catch (StorageException ex) {
               Console.WriteLine(String.Format("Failure: Status={0}({0:D}), Msg={1}",

                   (HttpStatusCode)ex.RequestInformation.HttpStatusCode,
                    ex.RequestInformation.HttpStatusMessage));
           }
       }





       /* The client library can automatically upload large blobs in blocks:
       // 1. Set # of blocks to simultaneously upload (range=1-64, default=1)
       client.ParallelOperationThreadCount = Environment.ProcessorCount;

       // 2. Define size of "large blob" (range=1MB-64MB, default=32MB)
       client.SingleBlobUploadThresholdInBytes = 32 * 1024 * 1024;

       // 3. Set individual block size (range=16KB-4MB, default=4MB)
       blockBlob.StreamWriteSizeInBytes = 4 * 1024 * 1024; */

       public static void PageBlobs(CloudStorageAccount account) {
           const Int32 c_BlobPageSize = 512;
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();
           CloudBlobContainer container = client.GetContainerReference("demo").EnsureExists();
           CloudPageBlob pageBlob = container.GetPageBlobReference("MyPageBlob.txt");
           pageBlob.DeleteIfExists();

           // You must create a page blob specifying its size:
           pageBlob.Create(10 * c_BlobPageSize);

           Byte[] data = new Byte[1 * c_BlobPageSize];  // Must be multiple of page size

           // Write some data to Page 0 (offset 0):
           Array.Copy(new Byte[] { 1, 2, 3, 4, 5 }, data, 5);
           pageBlob.WritePages(new MemoryStream(data), 0 * c_BlobPageSize); // Offset 0

           // Write some data to Page 2 (offset 1024):
           Array.Copy(new Byte[] { 5, 4, 3, 2, 1 }, data, 5);
           pageBlob.WritePages(new MemoryStream(data), 2 * c_BlobPageSize); // Offset 1024

           // Show committed pages:
           foreach (PageRange pr in pageBlob.GetPageRanges())
               Console.WriteLine("Start={0,6:N0}, End={1,6:N0}", pr.StartOffset, pr.EndOffset);

           // Read the whole blob (with lots of 0's):
           using (var stream = new MemoryStream()) {
               pageBlob.DownloadToStream(stream);
               data = stream.GetBuffer();
           }
           Console.WriteLine("Downloaded length={0:N0}", data.Length);
           Console.WriteLine("  Page 0 data: " + BitConverter.ToString(data, 0 * c_BlobPageSize, 10));
           Console.WriteLine("  Page 1 data: " + BitConverter.ToString(data, 1 * c_BlobPageSize, 10));
           Console.WriteLine("  Page 2 data: " + BitConverter.ToString(data, 2 * c_BlobPageSize, 10));

           // Read a specific range from the blob (offset 1024, 10 bytes):
           using (var stream = new MemoryStream()) {
               pageBlob.DownloadRangeToStream(stream, 2 * c_BlobPageSize, 10);
               stream.Seek(0, SeekOrigin.Begin);
               data = new BinaryReader(stream).ReadBytes((Int32)stream.Length);
               Console.WriteLine("  Page 2 data: " + BitConverter.ToString(data, 0, 10));
           }

           // Clear a range of pages (offset 0, 512 bytes):
           pageBlob.ClearPages(0, 512);

           // Show committed pages:
           foreach (PageRange pr in pageBlob.GetPageRanges())
               Console.WriteLine("Start={0,6:N0}, End={1,6:N0}", pr.StartOffset, pr.EndOffset);

           pageBlob.Delete();
       }

       public static void Segmented(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();

           // ListContainers/BlobsSegmented return up to 5,000 items
           const Int32 desiredItems = 5000 + 1000;  // Try to read more than the max

           // Create a bunch of blobs (this takes a very long time):
           CloudBlobContainer container = client.GetContainerReference("manyblobs");
           container.EnsureExists();
           for (Int32 i = 0; i < desiredItems; i++)
               container.GetBlockBlobReference(String.Format("{0:00000}", i)).UploadText("");

           // Enumerate containers & blobs in segments
           for (ContainerResultSegment crs = null; crs.HasMore(); ) {
               crs = client.ListContainersSegmented(null, ContainerListingDetails.None, desiredItems, crs.SafeContinuationToken());
               foreach (var c in crs.Results) {
                   Console.WriteLine("Container: " + c.Uri);
                   for (BlobResultSegment brs = null; brs.HasMore(); ) {
                       brs = c.ListBlobsSegmented(null, true, BlobListingDetails.None, desiredItems, brs.SafeContinuationToken(), null, null);
                       foreach (var b in brs.Results) Console.WriteLine("   Blob: " + b.Uri);
                   }
               }
           }
       }

       public static void Snapshots(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();
           CloudBlobContainer container = client.GetContainerReference("demo").EnsureExists(true);

           // Create the original blob:
           CloudBlockBlob origBlob = container.GetBlockBlobReference("Original.txt");
           origBlob.UploadText("Original data");

           // Create a snapshot of the original blob & save its timestamp:
           CloudBlockBlob snapshotBlob = origBlob.CreateSnapshot();
           DateTimeOffset? snapshotTime = snapshotBlob.SnapshotTime;

           // Try to write to the snapshot blob:
           try { snapshotBlob.UploadText("Fails"); }
           catch (InvalidOperationException ex) { Console.WriteLine(ex.Message); }

           // Modify the original blob & show it:
           origBlob.UploadText("New data");
           Console.WriteLine(origBlob.DownloadText());      // New data
           Console.WriteLine();

           // Show snapshot blob via original blob URI & snapshot time:
           snapshotBlob = container.GetBlockBlobReference("Original.txt", snapshotTime);
           Console.WriteLine(snapshotBlob.DownloadText());  // Original data

           // Show all blobs in the container with their snapshots:
           foreach (ICloudBlob b in container.ListBlobs(null, true, BlobListingDetails.Snapshots)) {
               Console.WriteLine("Uri={0}, Snapshot={1}", b.Uri, b.SnapshotTime);
           }

           // Create writable blob from the snapshot:
           CloudBlockBlob writableBlob = container.GetBlockBlobReference("Copy.txt");
           writableBlob.StartCopyFromBlob(snapshotBlob);
           Console.WriteLine(writableBlob.DownloadText());
           writableBlob.UploadText("Success");
           Console.WriteLine(writableBlob.DownloadText());

           // DeleteSnapshotsOption: None (blob only; throws StorageException if snapshots exist), IncludeSnapshots, DeleteSnapshotsOnly
           origBlob.Delete(DeleteSnapshotsOption.IncludeSnapshots);
       }

       public static void Leases(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient blobClient = account.CreateCloudBlobClient();
           CloudBlobContainer container = blobClient.GetContainerReference("demo").EnsureExists();
           container.SetPermissions(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Container });
           CloudBlockBlob blob = container.GetBlockBlobReference("test.txt");
           blob.UploadText("Here is some data");

           String leaseId = blob.AcquireLease(TimeSpan.FromSeconds(30), null);
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit(); // Succeeds: reads are OK while a lease is obtained

           // Try to acquire another lease:
           try { 
               leaseId = blob.AcquireLease(TimeSpan.FromSeconds(30), null);
           }
           catch (StorageException ex) {
               if ((HttpStatusCode)ex.RequestInformation.HttpStatusCode != HttpStatusCode.Conflict) throw;
               Console.WriteLine(ex.RequestInformation.HttpStatusMessage);
           }

           // Fails: Writes are not OK while a lease is held:
           try {
               blob.UploadText("Can't upload while lease held without lease ID");
           }
           catch (StorageException ex) {
               if ((HttpStatusCode)ex.RequestInformation.HttpStatusCode != HttpStatusCode.Conflict) throw;
               Console.WriteLine(ex.RequestInformation.HttpStatusMessage);
           }

           // Succeeds: Writes are OK if we specify a lease Id:
           try { 
               leaseId = blob.AcquireLease(TimeSpan.FromSeconds(30), null);
           }
           catch { } // Ensure we have the lease before doing the PUT

           blob.ReleaseLease(AccessCondition.GenerateLeaseCondition(leaseId));
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit();

           blob.UploadText("Data uploaded while lease NOT held");
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit();
       }


       public static void SignedAccessSignatures(CloudStorageAccount account) {
           Console.Clear();
           // Create a container (with no access) & upload a blob
           CloudBlobContainer container = account.CreateCloudBlobClient()
              .GetContainerReference("demo").EnsureExists();
           container.SetPermissions(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Off });
           CloudBlockBlob blob = container.GetBlockBlobReference("test.txt");
           blob.UploadText("Data accessed!");
           Process.Start("IExplore", blob.Uri.ToString()).WaitForExit(); // This fails

           // Create a SAS for the blob
           var start = DateTime.UtcNow.AddMinutes(-5); // -5 for clock skew
           SharedAccessBlobPolicy sabp = new SharedAccessBlobPolicy {
               SharedAccessStartTime = start,
               SharedAccessExpiryTime = start.AddHours(1),
               Permissions = SharedAccessBlobPermissions.Read |
                             SharedAccessBlobPermissions.Write |
                             SharedAccessBlobPermissions.Delete
           };
           String sas = blob.GetSharedAccessSignature(sabp);
           String sasUri = blob.Uri + sas;
           Process.Start("IExplore", sasUri); // This succeeds (in IE, modify URL and show failure)

           // Here's how we read the blob in code data via an SAS URI:
           CloudBlockBlob sasBlob = new CloudBlockBlob(blob.Uri, new StorageCredentials(sas));
           Console.WriteLine(sasBlob.DownloadText()); // This succeeds

           // Alternatively, we can add the SAP to the container with a name:
           String signatureIdentifier = "Revokable-" + DateTime.UtcNow;
           var permissions = container.GetPermissions();
           // NOTE: A container can have up to 5 SAPs policies
           permissions.SharedAccessPolicies.Add(signatureIdentifier,
              new SharedAccessBlobPolicy {
                  Permissions = SharedAccessBlobPermissions.Read |
                                SharedAccessBlobPermissions.Write
              });
           container.SetPermissions(permissions);

           // This SharedAccessPolicy CAN'T specify what is already present in the Signature Identifier
           sas = blob.GetSharedAccessSignature(new SharedAccessBlobPolicy {
               SharedAccessStartTime = start,
               SharedAccessExpiryTime = start.AddYears(10)
           }, signatureIdentifier);
           sasUri = blob.Uri + sas;
           Process.Start("IExplore", sasUri);

           // We can now revoke access on the container:
           permissions.SharedAccessPolicies.Remove(signatureIdentifier);
           container.SetPermissions(permissions);
           Process.Start("IExplore", sasUri); // This fails now
       }

       public static void DirectoryHierarchies(CloudStorageAccount account) {
           Console.Clear();
           CloudBlobClient client = account.CreateCloudBlobClient();
           Console.WriteLine("Default delimiter={0}", client.DefaultDelimiter /* settable */);
           Console.WriteLine();

           // Create the virtual directory
           const String virtualDirName = "demo";
           CloudBlobContainer virtualDir = client.GetContainerReference(virtualDirName).EnsureExists(true);

           // Create some file entries under the virtual directory
           String[] virtualFiles = new String[] {
				"FileA", "FileB", // Avoid  $&+,/:=?@ in blob names
				"Dir1/FileC", "Dir1/FileD", "Dir1/Dir2/FileE",
				"Dir3/FileF", "Dir3/FileG",
				"Dir4/FileH"
			};
           foreach (String file in virtualFiles) {
               virtualDir.GetBlockBlobReference("Root/" + file).UploadText(String.Empty);
           }

           // Show the blobs in the virtual directory container
           ShowContainerBlobs(virtualDir);   // Same as UseFlatBlobListing = false
           Console.WriteLine();
           ShowContainerBlobs(virtualDir, true);

           // CloudBlobDirectory (derived from IListBlobItem) is for traversing 
           // and accessing blobs with names structured in a directory hierarchy.
           CloudBlobDirectory root = virtualDir.GetDirectoryReference("Root");
           WalkBlobDirHierarchy(root, 0);

           // Show just the blobs under Dir1
           Console.WriteLine();
           String subdir = virtualDir.Name + "/Root/Dir1/";
           foreach (var file in client.ListBlobs(subdir))
               Console.WriteLine(file.Uri);
       }

       private static void ShowContainerBlobs(CloudBlobContainer container, Boolean useFlatBlobListing = false, BlobListingDetails details = BlobListingDetails.None, BlobRequestOptions options = null, OperationContext operationContext = null) {
           Console.WriteLine("Container: " + container.Name);
           for (BlobResultSegment brs = null; brs.HasMore(); ) {
               brs = container.ListBlobsSegmented(null, useFlatBlobListing, details, 1000, brs.SafeContinuationToken(), options, operationContext);
               foreach (var blob in brs.Results) Console.WriteLine("   " + blob.Uri);
           }
       }

       private static void WalkBlobDirHierarchy(CloudBlobDirectory dir, Int32 indent) {
           // Get all the entries in the root directory
           IListBlobItem[] entries = dir.ListBlobs().ToArray();
           String spaces = new String(' ', indent * 3);

           Console.WriteLine(spaces + dir.Prefix + " entries:");
           foreach (var entry in entries.OfType<ICloudBlob>())
               Console.WriteLine(spaces + "   " + entry.Name);

           foreach (var entry in entries.OfType<CloudBlobDirectory>()) {
               String[] segments = entry.Uri.Segments;
               CloudBlobDirectory subdir = dir.GetSubdirectoryReference(segments[segments.Length - 1]);
               WalkBlobDirHierarchy(subdir, indent + 1); // Recursive call
           }
       }

       private static void ShowContainer(CloudBlobContainer container, Boolean showBlobs) {
           Console.WriteLine("Blob container={0}", container);

           BlobContainerPermissions permissions = container.GetPermissions();
           String[] meanings = new String[] {
				"no public access", 
				"anonymous clients can read container & blob data", 
				"anonymous readers can read blob data only"
			};
           Console.WriteLine("Container's public access={0} ({1})",
              permissions.PublicAccess, meanings[(Int32)permissions.PublicAccess]);

           // Show collection of access policies; each consists of name & SharedAccesssPolicy
           // A SharedAccesssBlobPolicy contains:
           //    SharedAccessPermissions enum (None, Read, Write, Delete, List) & 
           //    SharedAccessStartTime/SharedAccessExpireTime
           Console.WriteLine("   Shared access policies:");
           foreach (var policy in permissions.SharedAccessPolicies) {
               Console.WriteLine("   {0}={1}", policy.Key, policy.Value);
           }

           container.FetchAttributes();
           Console.WriteLine("   Attributes: Name={0}, Uri={1}", container.Name, container.Uri);
           Console.WriteLine("   Properties: LastModified={0}, ETag={1},", container.Properties.LastModified, container.Properties.ETag);
           ShowMetadata(container.Metadata);

           if (showBlobs)
               foreach (ICloudBlob blob in container.ListBlobs())
                   ShowBlob(blob);
       }

       private static void ShowBlob(ICloudBlob blob) {
           // A blob has attributes: Uri, Snapshot DateTime?, Properties & Metadata
           // The CloudBlob Uri/SnapshotTime/Properties/Metadata properties return these
           // You can set the properties & metadata; not the Uri or snapshot time
           Console.WriteLine("Blob Uri={0}, Snapshot time={1}", blob.Uri, blob.SnapshotTime);
           BlobProperties bp = blob.Properties;
           Console.WriteLine("BlobType={0}, CacheControl={1}, Encoding={2}, Language={3}, MD5={4}, ContentType={5}, LastModified={6}, Length={7}, ETag={8}",
              bp.BlobType, bp.CacheControl, bp.ContentEncoding, bp.ContentLanguage, bp.ContentMD5, bp.ContentType, bp.LastModified, bp.Length, bp.ETag);
           ShowMetadata(blob.Metadata);
       }

       private static void ShowMetadata(IDictionary<String, String> metadata) {
           foreach (var kvp in metadata)
               Console.WriteLine("{0}={1}", kvp.Key, kvp.Value);
       }
   }

   private static class TablePatterns {

       public static void Basics(CloudStorageAccount account) {
           Console.Clear();

           //// Use an OperationContext for debugging and to estimate billing
           //OperationContext oc = new OperationContext();
           //oc.SendingRequest += (Object s, RequestEventArgs e) => {
           //    HttpWebRequest request = e.Request;
           //};
           //oc.ResponseReceived += (Object s, RequestEventArgs e) => {
           //    HttpWebRequest request = e.Request;
           //    HttpWebResponse response = e.Response;
           //    RequestResult rr = e.RequestInformation;
           //};
           

           CloudTableClient client = account.CreateCloudTableClient();
           //client.RetryPolicy = new ExponentialRetry();

           CloudTable ct = client.GetTableReference(c_tableName).Clear();

           // Create 2 insert table operations & execute them:
           var to = TableOperation.Insert(new CustomerEntity("Jeff", "C_Jeff") { Name = "Jeff", City = "Seattle" });
           TableResult tr = ct.Execute(to);
           to = TableOperation.Insert(new CustomerEntity("Paul", "C_Paul") { Name = "Paul", City = "Olympia" });
           tr = ct.Execute(to);

           foreach (var pk in new[] { "Jeff", "Paul" }) {
               // Read all entities where PartitionKey == pk
               TableQuery<CustomerEntity> query = new TableQuery<CustomerEntity> {
                   FilterString = new FilterBuilder<CustomerEntity>(ce => ce.PartitionKey, CompareOp.EQ, pk),
                   SelectColumns = new[] { new PropertyName<CustomerEntity>(ce => ce.Name).ToString() },
                   TakeCount = null
               };

               foreach (var customer in ct.ExecuteQuery(query)) {
                   Console.WriteLine(customer);
                   if (customer.Name == "Jeff") {      // Update "Jeff"
                       customer.Name = customer.Name.ToUpper();
                       //tr = ct.Execute(TableOperation.Replace(customer)); // HTTP PUT update
                       tr = ct.Execute(TableOperation.Merge(customer));     // HTTP MERGE update
                   }
                   else {
                       tr = ct.Execute(TableOperation.Delete(customer));  // Delete "Paul"            
                   }
               }
           }
       }

       public static void QueryFilterStrings(CloudStorageAccount account) {
           Console.Clear();

           CloudTable ct = account.CreateCloudTableClient().GetTableReference(c_tableName);
           const String pk = "Jeff", rk = "C_Jeff";

           // Best performance: Query single entity via specific PK/RK
           // This syntax puts the PK & RK in the URL itself
           var q1 = new TableQuery {
               FilterString = new FilterBuilder<TableEntity>
                   (te => te.PartitionKey, CompareOp.EQ, pk).And
                       (te => te.RowKey, CompareOp.EQ, rk)
           };
           TableQuery tq1 = new TableQuery() { FilterString = q1.FilterString };
           var r1 = ct.ExecuteQuery(tq1).ToList();

           // and again using the TableOperation.Retrieve method
           var r1b = ct.Execute(TableOperation.Retrieve<TableEntity>(pk, rk)).Result;

           //PK and RK in the FilterString
           var q2 = new TableQuery {
               FilterString = new FilterBuilder<TableEntity>
                   (te => te.PartitionKey, CompareOp.EQ, pk).And
                       (te => te.RowKey, CompareOp.EQ, rk)
           };
           TableQuery tq2 = new TableQuery() { FilterString = q2.FilterString };
           var r2 = ct.ExecuteQuery(tq2).ToList();

           // Worst performance: Query against whole table because no PK or RK provided
           var q3 = new TableQuery { FilterString = "City eq 'Redmond'" };
           TableQuery tq3 = new TableQuery() { FilterString = q3.FilterString };
           var r3 = ct.ExecuteQuery(tq3).ToList();

           // medium performance: query a range of partition keys
           var q4 = new TableQuery { FilterString = new FilterBuilder<TableEntity>
               (te => te.PartitionKey, RangeOp.IncludeLowerExcludeUpper, pk) };
           TableQuery tq4 = new TableQuery() { FilterString = q4.FilterString };
           var r4 = ct.ExecuteQuery(tq4).ToList();


           String s;
           TableQuery tq;
           List<DynamicTableEntity> r;

           s = TableQuery.GenerateFilterConditionForBool("ABoolean", "eq", true);
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForBinary("AByteArray", "eq", new Byte[] { 1, 2, 3, 4, 5 });
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForDate("ADate", "eq", DateTimeOffset.Now);
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForDouble("ADouble", "eq", 1.23);
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForGuid("AGuid", "eq", Guid.NewGuid());
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForInt("AnInt32", "eq", 123);
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

           s = TableQuery.GenerateFilterConditionForLong("AnInt64", "eq", 321);
           tq = new TableQuery { FilterString = s };
           r = ct.ExecuteQuery(tq).ToList();

       }

       public static void OptimisticConcurrency(CloudStorageAccount account) {
           Console.Clear();
           // Preparation: Clear the table and add a customer to it
           CloudTableClient client = account.CreateCloudTableClient();
           CloudTable ct = client.GetTableReference(c_tableName).Clear();
           ct.Execute(TableOperation.Insert(new CustomerEntity("Jeff", "Richter") { Name = "Jeff", City = "Seattle" }));

           // 1st client queries the entity, updates it, & saves it back
           ThreadPool.QueueUserWorkItem(o => OptimisticUpdatePattern(client, "Alpha"));

           // 2nd client also queries the entity, updates it, & saves it back
           OptimisticUpdatePattern(client, "Beta");

           ShowFirstEntity(client, c_tableName);
       }

       private static void OptimisticUpdatePattern(CloudTableClient client, String newCity) {
           const String title = "Optimistic Concurrency Pattern";
           CloudTable ct = client.GetTableReference(c_tableName);

           // Read the first entity from the table:
           CustomerEntity ce = ct.ExecuteQuery(new TableQuery<CustomerEntity>()).First();
           ce.City = newCity;   // Change a property's value
           while (true) {
               try {
                   MessageBox.Show(
                      String.Format("NewCity={0}\r\nETag={1}\r\nHit OK to save changes", newCity, ce.ETag),
                      title, MessageBoxButtons.OK);
                   Debugger.Break();
                   ct.Execute(TableOperation.Replace(ce));
                   break;   // No exception, success!
               }
               catch (StorageException ex) {
                   if ((HttpStatusCode)ex.RequestInformation.HttpStatusCode != HttpStatusCode.PreconditionFailed) throw;
                   MessageBox.Show(
                      String.Format("Failed to save {0}\r\nHit OK to try again", newCity),
                      title, MessageBoxButtons.OK);
                   // Reread the entity to update its ETag; the changed props don't change because of PreserveChanges
                   ce = (CustomerEntity)ct.Execute(TableOperation.Retrieve<CustomerEntity>(ce.PartitionKey, ce.RowKey)).Result;
               }
           }
       }

       // JMR: null properties are not persisted in the table
       public static void LastUpdateWins(CloudStorageAccount account) {
           Console.Clear();
           // Preparation: Clear the table and add a customer to it
           CloudTableClient client = account.CreateCloudTableClient();
           var ct = client.GetTableReference(c_tableName).Clear();
           ct.Execute(TableOperation.Insert(new CustomerEntity("Jeff", "Richter") { Name = "Jeff", City = "Seattle" }));

           // Last update always wins
           CustomerEntity ce1 = ct.ExecuteQuery(new TableQuery<CustomerEntity> { FilterString = String.Empty }).First();
           ce1.City = "Alpha";
           ce1.ETag = "*";   // "*" ETag matches whatever ETag the entity in the table has

           CustomerEntity ce2 = ct.ExecuteQuery(new TableQuery<CustomerEntity> { FilterString = String.Empty }).First();
           ce2.City = "Beta";
           ce2.ETag = "*";   // "*" ETag matches whatever ETag the entity in the table has
           ct.Execute(TableOperation.Replace(ce1));  // Last update wins
           ShowFirstEntity(client, c_tableName);

           ct.Execute(TableOperation.Replace(ce2));  // Last update wins
           ShowFirstEntity(client, c_tableName);
       }

       private static void ShowFirstEntity(CloudTableClient client, String tableName) {
           Console.WriteLine(client.GetTableReference(tableName).ExecuteQuery(new TableQuery<CustomerEntity>()).First());
       }

       public static void Segmented(CloudStorageAccount account) {
           Console.Clear();
           const Int32 c_maxEntitiesPerBatch = 100;
           const String tableName = "demoSegmented";
           CloudTableClient client = account.CreateCloudTableClient();
           var ct = client.GetTableReference(tableName);

           if (client.GetTableReference(tableName).CreateIfNotExists()) {
               // The table didn't exist, populate it
               TableBatchOperation tbo = null;
               for (Int32 i = 0; i < 2300; i++) {
                   if ((i % c_maxEntitiesPerBatch) == 0) {
                       // Batch save all the entities 
                       if (tbo != null) ct.ExecuteBatch(tbo);
                       tbo = new TableBatchOperation();
                   }
                   tbo.Add(TableOperation.Insert(new CustomerEntity("PK", String.Format("Cust{0:00000}", i))));
               }
               ct.ExecuteBatch(tbo);
           }

           // Enumerate table entities (ExecuteQuerySegmented returns up to 1,000 items)
           // NOTE: HasMore & SafeContinuationToken
           for (TableQuerySegment<CustomerEntity> tqs = null; tqs.HasMore(); ) {
               tqs = ct.ExecuteQuerySegmented(new TableQuery<CustomerEntity> { FilterString = String.Empty }, tqs.SafeContinuationToken());
               foreach (var entity in tqs.Results) Console.WriteLine(entity.RowKey);
           }

           // Enumerate table names (ListTablesSegmented returns up to 1,000 items)
           for (TableResultSegment trs = null; trs.HasMore(); ) {
               trs = client.ListTablesSegmented(trs.SafeContinuationToken());
               foreach (var tablename in trs.Results) Console.WriteLine(tablename);
           }
       }

       public static void MultipleKinds(CloudStorageAccount account) {
           Console.Clear();
           CloudTable ct = account.CreateCloudTableClient().GetTableReference("CustomerAndOrders").EnsureExists(true);
           // Add a single entity with one PartitionKey value
           ct.Execute(TableOperation.Insert(new CustomerEntity("Paul", "C_Paul") { Name = "Paul", City = "Olympia" }));

           // Add many entities sharing the same PartitionKey value
           TableBatchOperation tbo = new TableBatchOperation();
           tbo.Add(TableOperation.Insert(new CustomerEntity("Jeff", "C_Jeff") {
               Name = "Jeff",
               City = "Seattle"
           }));
           tbo.Add(TableOperation.Insert(new OrderEntity("Jeff", "O_Milk") {
               Item = "Milk",
               Quantity = 2 
           }));
           tbo.Add(TableOperation.Insert(new OrderEntity("Jeff", "O_Cereal") {
               Item = "Cereal",
               Quantity = 1
           }));
           ct.ExecuteBatch(tbo);

           IEnumerable<DynamicTableEntity> jeffEntities = ct.ExecuteQuery(
              new TableQuery { FilterString =
                  new FilterBuilder<TableEntity>(te => te.PartitionKey, CompareOp.EQ, "Jeff") });

           tbo = new TableBatchOperation();
           foreach (var e in jeffEntities) {
               switch ((EntityKindCodes)e.Properties["KindCode"].Int32Value) { // What kind of entity is this?
                   case EntityKindCodes.Customer:
                       var customer = new CustomerEntity(e.PartitionKey, e.RowKey) { ETag = e.ETag };
                       customer.ReadEntity(e.Properties, null);
                       customer.City = "Philadelphia";
                       tbo.Add(TableOperation.Replace(customer));
                       break;
                   case EntityKindCodes.Order:
                       var order = new OrderEntity(e.PartitionKey, e.RowKey) { ETag = e.ETag };
                       order.ReadEntity(e.Properties, null);
                       if (order.Item == "Milk") {
                           order.Quantity = 5;
                           tbo.Add(TableOperation.Replace(order));
                       }
                       break;
               }
           }
           ct.ExecuteBatch(tbo);
       }

       private static void ShowAllCustomersAndOrders(CloudTable ct) {
           // Query all customers & each customer's orders:
           var customerQuery = new TableQuery<CustomerEntity> {
               FilterString = new FilterBuilder<CustomerEntity>
                   (ce => ce.KindCode, CompareOp.EQ, (Int32)EntityKindCodes.Customer)
           };
           var orderQuery = new TableQuery<OrderEntity> {
               FilterString = new FilterBuilder<OrderEntity>
                   (oe => oe.KindCode, CompareOp.EQ, (Int32)EntityKindCodes.Order)
           };

           foreach (var ce in ct.ExecuteQuery(customerQuery)) {
               Console.WriteLine("Customer: " + ce);
               foreach (var oe in ct.ExecuteQuery(orderQuery)) Console.WriteLine("   Order: " + oe);
               Console.WriteLine();
           }
       }

#if false
      // http://blogs.msdn.com/b/windowsazurestorage/archive/2012/06/12/introducing-table-sas-shared-access-signature-queue-sas-and-update-to-blob-sas.aspx
      public static void SignedAccessSignatures(CloudStorageAccount account) {
         Console.Clear();
         // Create a table
         var ct = new ACloudTable(account.CreateCloudTableClient(), "sasTable").EnsureExists(true);

         // Create a SAS for the table from StartPK/StartRK to EndPK/EndRK
         SharedAccessTablePolicy policy = new SharedAccessTablePolicy {
            SharedAccessExpiryTime = DateTime.UtcNow.AddHours(1),
            Permissions = SharedAccessTablePermissions.Add
                | SharedAccessTablePermissions.Query
                | SharedAccessTablePermissions.Update
                | SharedAccessTablePermissions.Delete
         };

         String sas = ct.GetSharedAccessSignature(
             policy,    // access policy
             null,      // access policy identifier
             "StartPK", // start partition key
             null,      // start row key
             "EndPK",   // end partition key
             null);     // end row key

         StorageCredentials creds = new StorageCredentialsSharedAccessSignature(sas);
         var table = new ACloudTable(new CloudTableClient(account.TableEndpoint, creds), "sasTable");
         // Perform CRUD oerations here:

         String sasUri = ct.Uri + sas;
         ShowIE(sasUri); // This succeeds (in IE, modify URL and show failure)

         // Here's how we read the blob in code data via an SAS URI:
         CloudBlob sasBlob = new CloudBlob(sasUri);
         Console.WriteLine(sasBlob.DownloadText()); // This succeeds

         // Alternatively, we can add the SAP to the container with a name:
         String signatureIdentifier = "Revokable-" + DateTime.UtcNow;
         TablePermissions permissions = new TablePermissions();
         SharedAccessTablePolicy policy = new SharedAccessTablePolicy {
            SharedAccessExpiryTime = DateTime.MaxValue,
            Permissions = SharedAccessTablePermissions.Query | SharedAccessTablePermissions.Delete
         };
         permissions.SharedAccessPolicies.Add(signatureIdentifier, policy);
    table.SetPermissions(permissions);
 
    sas = table.GetSharedAccessSignature(new SharedAccessTablePolicy(), signatureIdentifier,
        null, null, // start partition key & row key
        null, null); // end partition key & row key
 
    // Create the GC queue SAS policy.
    QueuePermissions qPermissions = new QueuePermissions();
    SharedAccessQueuePolicy qPolicy = new SharedAccessQueuePolicy {
        SharedAccessExpiryTime = DateTime.MaxValue,
        Permissions = SharedAccessQueuePermissions.ProcessMessages
    };
 
    // Associate the above policy with a signed identifier
    qPermissions.SharedAccessPolicies.Add(signatureIdentifier, qPolicy);
 
queue.SetPermissions(qPermissions);
 
    sas = q.GetSharedAccessSignature(new SharedAccessQueuePolicy(), signatureIdentifier);
}
 

         var permissions = table.GetPermissions();
         // NOTE: A container can have up to 5 SAPs policies
         permissions.SharedAccessPolicies.Add(signatureIdentifier,
            new SharedAccessPolicy {
               Permissions = SharedAccessPermissions.Read |
                             SharedAccessPermissions.Write
            });
         container.SetPermissions(permissions);

         // This SharedAccessPolicy CAN'T specify what is already present in the Signature Identifier
         sas = blob.GetSharedAccessSignature(new SharedAccessPolicy {
            SharedAccessStartTime = start, SharedAccessExpiryTime = start.AddYears(10)
         }, c_signatureIdentifier);
         sasUri = blob.Uri + sas;
         Process.Start("IExplore", sasUri);

         // We can now revoke access on the container:
         permissions.SharedAccessPolicies.Remove(c_signatureIdentifier);
         container.SetPermissions(permissions);
         Process.Start("IExplore", sasUri); // This fails now
      }
#endif

       public sealed class SchemaV1 : TableEntity {
           public SchemaV1() { }
           public SchemaV1(String pk, String rk) : base(pk, rk) { Version = 1; }
           public Int32 Version { get; set; }
           public String PropertyV1 { get; set; }
           public override String ToString() {
               return String.Format("PK={0}, RK={1}, Version={2}, PropertyV1={3}",
                  PartitionKey, RowKey, Version, PropertyV1);
           }
       }

       public sealed class SchemaV2 : TableEntity {
           public SchemaV2(String pk, String rk) : base(pk, rk) { Version = 2; }
           public SchemaV2() { }
           public Int32 Version { get; set; }
           public String PropertyV1 { get; set; }
           public String PropertyV2 { get; set; }
           public override String ToString() {
               return String.Format("PK={0}, RK={1}, Version={2}, PropertyV1={3}, PropertyV2={4}",
                  PartitionKey, RowKey, Version, PropertyV1, PropertyV2);
           }
       }

       public static void SchemaVersioning(CloudStorageAccount account) {
           Console.Clear();
           CloudTable ct = account.CreateCloudTableClient().GetTableReference("versionTable").EnsureExists(true);

           // Add some entities with V1 schema to a table
           ct.Execute(TableOperation.Insert(new SchemaV1("PK", "RK1") { PropertyV1 = "V1" }));
           ct.Execute(TableOperation.Insert(new SchemaV1("PK", "RK2") { PropertyV1 = "V1" }));

           // App-V2 reads SchemaV1 and converts to SchemaV2 (added property)
           // Missing properties (eg: PropertyV2) will have default values
           IEnumerable<SchemaV2> entitiesV2 = ct.ExecuteQuery(new TableQuery<SchemaV2> { FilterString = String.Empty });

           // Convert from SchemaV1 to SchemaV2
           foreach (var e in entitiesV2) {
               e.Version = 2;
               e.PropertyV2 = "V2";
               // Don't use [InsertOr]Replace: it will delete properties that App-V1 still needs
               ct.Execute(TableOperation.InsertOrMerge(e));
           }

           // Have App-V1 read Schema-V2 entities
           IEnumerable<SchemaV1> entitiesV1 = ct.ExecuteQuery(new TableQuery<SchemaV1> { FilterString = String.Empty });
           foreach (var e in entitiesV1) {
               // App-V1 can still modify properties it knows about
               e.PropertyV1 = "Updated property";
               ct.Execute(TableOperation.InsertOrReplace(e));
               Console.WriteLine(e);
           }
       }

       #region Table Entity Definitions: Application-specific types
       // 1. Define entity kinds in this enum
       private enum EntityKindCodes {
           // NOTE: Explicitly assign values to these symbols and never change them!
           Customer = 0,
           Order = 1
       }

       // 2. Derive 1 type per enum symbol from this base class; pass the enum type for TKindEnum
       public class KindTableEntity<TKindEnum> : TableEntity {
           public KindTableEntity() : base() { }
           public KindTableEntity(String partitionKey, String rowKey, TKindEnum kindCode)
               : base(partitionKey, rowKey) {
               KindCode = (Int32)(Object)kindCode;
           }
           public Int32 KindCode { get; set; }
           public TKindEnum GetKindCode() { return (TKindEnum)(Object)KindCode; }
           public override String ToString() {
               return String.Format("PartitionKey={0}, RowKey={1}, Kind={2}", PartitionKey, RowKey, GetKindCode());
           }
       }

       // 3a. This is the Entity for Customers
       private sealed class CustomerEntity : KindTableEntity<EntityKindCodes> {
           public CustomerEntity() : this(null, null) { }
           public CustomerEntity(String partitionKey, String rowKey) : base(partitionKey, rowKey, EntityKindCodes.Customer) { }
           public String Name { get; set; }
           public String City { get; set; }
           public override String ToString() {
               return String.Format("{0}, Name={1}, City={2}", base.ToString(), Name, City);
           }
       }

       // 3b. This is the Entity for Orders
       private sealed class OrderEntity : KindTableEntity<EntityKindCodes> {
           public OrderEntity() : this(null, null) { }
           public OrderEntity(String partitionKey, String rowKey) : base(partitionKey, rowKey, EntityKindCodes.Order) { }
           public String Item { get; set; }
           public Int32 Quantity { get; set; }
           public override String ToString() {
               return String.Format("{0}, Item={1}, Quantity={2}", base.ToString(), Item, Quantity);
           }
       }
       #endregion
   }

   private static class QueuePatterns {
       public static void Basics(CloudStorageAccount account) {
           Console.Clear();
           CloudQueueClient client = account.CreateCloudQueueClient();
           CloudQueue queue = CreateQueue(client);
           Thread.Sleep(5000);
           AddMessages(queue);
           GetApproximateMessageCount(queue);
           PeekMessages(queue);
           RetrieveMessages(queue);
           UpdateMessages(queue);
           DeleteMessages(queue);
           queue.Delete();
       }
       private static CloudQueue CreateQueue(CloudQueueClient client) {
           CloudQueue queue = client.GetQueueReference("demo").EnsureExists(true);
           return queue;
       }
       private static void AddMessages(CloudQueue queue) {
           // Add 2 messages to a queue:
           var timeToLive = TimeSpan.FromDays(4);
           queue.AddMessage(new CloudQueueMessage("MsgA"), timeToLive);
           queue.AddMessage(new CloudQueueMessage("MsgB"), timeToLive);
       }
       private static void GetApproximateMessageCount(CloudQueue queue) {
           queue.FetchAttributes();   // Call this 1st to get ApproximateMesageCount
           Console.WriteLine("There are approximately {0} messages in the queue.", queue.ApproximateMessageCount);
       }
       private static void PeekMessages(CloudQueue queue) {
           const Int32 c_MaxMsgsToGet = 32;
           Console.WriteLine("Peek the messages:");
           foreach (var cqm in queue.PeekMessages(c_MaxMsgsToGet))
               Console.WriteLine(QueueMessageToString(cqm));
           Console.WriteLine();
       }
       private static void RetrieveMessages(CloudQueue queue) {
           // Consumers process messages:
           const Int32 c_MaxMsgsToGet = 32;
           var visibilityTimeout = TimeSpan.FromSeconds(5); // Max=7 days
           Int32 demo = 0;
           while (true) {
               var msgs = queue.GetMessages(c_MaxMsgsToGet, visibilityTimeout);
               if (msgs.Count() == 0) {
                   Thread.Sleep(1000);  // IMPORTANT: Balance cost with latency
                   continue;
               }
               foreach (CloudQueueMessage cqm in msgs) {
                   Console.WriteLine(QueueMessageToString(cqm)); // For demo
                   if (cqm.DequeueCount < 3) {
                       // Process message here...
                   }
                   else {
                       // Log poison msg to fix code here...
                   }
                   if (demo == 1) {
                       //replace the content of the message with new content
                       cqm.SetMessageContent("new content");
                       queue.UpdateMessage(cqm, new TimeSpan(0, 0, 30), MessageUpdateFields.Visibility | MessageUpdateFields.Content);

                       // extend the message another 30 seconds
                       queue.UpdateMessage(cqm, new TimeSpan(0, 0, 30), MessageUpdateFields.Visibility);

                       //// delete the message
                       //queue.SafeDeleteMessage(cqm);
                   }
               }
               if (demo++ == 1) break;
               Console.WriteLine();
           }
       }
       private static void UpdateMessages(CloudQueue queue) {
           // Consumers process messages:
           const Int32 c_MaxMsgsToGet = 32;
           var visibilityTimeout = TimeSpan.FromSeconds(5); // Max=7 days
           Int32 demo = 0;
           while (true) {
               var msgs = queue.GetMessages(c_MaxMsgsToGet, visibilityTimeout);
               if (msgs.Count() == 0) {
                   Thread.Sleep(1000);  // IMPORTANT: Balance cost with latency
                   continue;
               }
               foreach (CloudQueueMessage cqm in msgs) {
                   Console.WriteLine(QueueMessageToString(cqm)); // For demo
                   if (cqm.DequeueCount < 3) {
                       // Process message here...
                   }
                   else {
                       // Log poison msg to fix code here...
                   }
                   if (demo == 1) {
                       //replace the content of the message with new content
                       cqm.SetMessageContent("new content");
                       queue.UpdateMessage(cqm, new TimeSpan(0, 0, 30), MessageUpdateFields.Visibility | MessageUpdateFields.Content);

                       // extend the message another 30 seconds
                       queue.UpdateMessage(cqm, new TimeSpan(0, 0, 30), MessageUpdateFields.Visibility);

                       // delete the message
                       queue.SafeDeleteMessage(cqm);
                   }
               }
               if (demo++ == 1) break;
               Console.WriteLine();
           }
       }
       private static void DeleteMessages(CloudQueue queue) {
           // Consumers process messages:
           const Int32 c_MaxMsgsToGet = 32;
           var visibilityTimeout = TimeSpan.FromSeconds(5); // Max=7 days
           while (true) {
               var msgs = queue.GetMessages(c_MaxMsgsToGet, visibilityTimeout);
               if (msgs.Count() == 0) {
                   Thread.Sleep(1000);  // IMPORTANT: Balance cost with latency
                   continue;
               }
               foreach (CloudQueueMessage cqm in msgs) {
                   Console.WriteLine(QueueMessageToString(cqm)); // For demo
                    // delete the message
                    queue.SafeDeleteMessage(cqm);
               }
           }
       }




       private static String QueueMessageToString(CloudQueueMessage cqm) {
           var sb = new StringBuilder();
           sb.AppendLine("Id:              " + cqm.Id);
           sb.AppendLine("InsertionTime:   " + cqm.InsertionTime);
           sb.AppendLine("ExpirationTime:  " + cqm.ExpirationTime);
           sb.AppendLine("NextVisibleTime: " + cqm.NextVisibleTime);
           sb.AppendLine("DequeueCount:    " + cqm.DequeueCount);
           sb.AppendLine("PopReceipt:      " + cqm.PopReceipt);
           sb.AppendLine("Message text:    " + cqm.AsString);
           return sb.ToString();
       }

       public static void Segmented(CloudStorageAccount account) {
           CloudQueueClient client = account.CreateCloudQueueClient();
           for (QueueResultSegment qrs = null; qrs.HasMore(); ) {
               qrs = client.ListQueuesSegmented(qrs.SafeContinuationToken());
               foreach (var q in qrs.Results)
                   Console.WriteLine(q.Uri);
           }
       }
   }

   static AzureBookStoragePatterns() {
       String[] args = Environment.GetCommandLineArgs();
       if (args.Length == 1) { // This instance run by a user, spawn the storage killer
           while (!Process.GetProcessesByName("DSServiceLDB").Any()) {
               try {
                   var psi = new ProcessStartInfo(@"C:\Program Files\Microsoft SDKs\Windows Azure\Emulator\CSRun", "/devstore:start") { WindowStyle = ProcessWindowStyle.Hidden };
                   Process.Start(psi);
                   break;   // Started ok, break out
               }
               catch (System.ComponentModel.Win32Exception ex) {
                   var e = ex;
                   MessageBox.Show("Start the Windows Azure Storage Emulator");
               }
           }

           if (c_SpawnStorageKiller) {
               var psi = new ProcessStartInfo(args[0], Process.GetCurrentProcess().Id.ToString()) { WindowStyle = ProcessWindowStyle.Minimized };
               Process.Start(psi);
           }
       }
       else { // This instance is the storage killer
           Process p = Process.GetProcessById(Int32.Parse(args[1]));
           Console.BackgroundColor = ConsoleColor.DarkRed;
           Console.Clear();
           Console.Title = "WADS demo storage killer (waiting for demo app to terminate)";
           p.WaitForExit();
           Console.WriteLine("Press Ctrl-C to keep storage or any other key to delete now.");
           var stop = DateTime.Now + TimeSpan.FromSeconds(30);
           while (DateTime.Now < stop && !Console.KeyAvailable) {
               Console.Title = String.Format("Deleting storage in {0} second(s).",
                  (Int32)((stop - DateTime.Now).TotalSeconds));
               Thread.Sleep(400);
           }

           DeleteDemoStorage(GetStorageAccount(StorageAccountType.DevStorage));
           DeleteDemoStorage(GetStorageAccount(StorageAccountType.AzureStorage));
           Console.WriteLine(); 
           Console.WriteLine("Storage deleted");
           Thread.Sleep(5000); // Let the user see the deleted storage objects
           Environment.Exit(0);
       }
   }

   private static void DeleteDemoStorage(CloudStorageAccount account) {
       // Blobs
       CloudBlobClient blobClient = account.CreateCloudBlobClient();
       CloudBlobContainer rootContainer = blobClient.GetContainerReference("$root");
       try {
           rootContainer.Delete();
           Console.WriteLine("Deleting blob container: " + rootContainer.Uri);
       }
       catch (StorageException) { }
       foreach (var c in blobClient.ListContainers().Where(container => container.Name.StartsWith("demo"))) {
           Console.WriteLine("Deleting blob container: " + c.Uri);
           c.Delete();
       }

       // Tables
       CloudTableClient tableClient = account.CreateCloudTableClient();
       foreach (var table in tableClient.ListTables().Where(ct => ct.Name.StartsWith("demo"))) {
           Console.WriteLine("Deleting table: " + table.Name);
           table.Delete();
       }

       // Queues
       CloudQueueClient queueClient = account.CreateCloudQueueClient();
       foreach (var queue in queueClient.ListQueues().Where(q => q.Name.StartsWith("demo"))) {
           Console.WriteLine("Deleting queue: " + queue.Uri);
           queue.Delete();
       }
   }

   #region Show Methods
   //[DebuggerStepThrough]
   //private static void Console.Clear() { Console.Clear() }
   //private struct NewLines {
   //   [DebuggerStepThrough]
   //   public void NL(Int32 newLines = 1) {
   //      for (; newLines > 0; newLines--) Console.WriteLine();
   //   }
   //}

   //[DebuggerStepThrough]
   //private static NewLines Console.WriteLine(Object arg1 = null) {
   //   Console.WriteLine(arg1); return new NewLines();
   //}

   //[DebuggerStepThrough]
   //private static NewLines Console.WriteLine(String format, params Object[] args) {
   //   Console.WriteLine(format, args);
   //   return new NewLines();
   //}

   //[DebuggerStepThrough]
   //private static void ShowIE(Object uri) {
   //   Process.Start("IExplore", uri.ToString()).WaitForExit();
   //}
   #endregion


}

#if FACEBOOK
internal static class Facebook {
    /*
     * User table: UserId(RK), name(PK), & other info
     * Friend table: UserId (user,PK) -> UserId (friends, RK) [repeat for each friend]
     * Status table: UserId(PK) -> text (RK)
     */
    private sealed class User : TableServiceEntity {
        public User(String id, String name) : base(name, id) { Id = id; Name = name; }
        public String Id { get; set; }
        public String Name { get; set; }
    }
    private sealed class Friend : TableServiceEntity {
        public Friend(String userId, String friendId) : base(userId, friendId) {
            UserId = userId;
            FriendId = friendId;
        }
        public String UserId { get; set; }
        public String FriendId { get; set; }
    }
    private sealed class Status : TableServiceEntity {
        public Status(String userId, String text)
            : base(userId, DateTime.UtcNow.ToString()/* TODO: reverse*/) {
                UserId = userId;
                When = DateTime.UtcNow;
                StatusText = text;
        }
        public String UserId { get; set; }
        public DateTime When { get; set; }
        public String StatusText { get; set; }
    }
}
#endif
